# -*- coding: utf-8 -*-
"""Provides utilities to help multi-site aware applications.

:Authors:
    - Bruce Kroeze
"""
"""
New BSD License
===============
Copyright (c) 2008, Bruce Kroeze http://coderseye.com

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of SolidSiteSolutions LLC, Zefamily LLC nor the names of its 
      contributors may be used to endorse or promote products derived from this 
      software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
__docformat__="restructuredtext"

from threaded_multihost import threadlocals
from django.contrib.sites.models import Site
from satchmo import caching
import logging

log = logging.getLogger('threaded_multihost.sites')
_WARNED = False

def site_by_host(host=None):
    """Get the current site by looking at the request stored in the thread.  
    
    Returns the best match found in the `django.contrib.sites` app.  If not
    found, then returns the default set as given in `settings.SITE_ID`
    """
    site = None

    if not host:
        request = threadlocals.get_current_request()
        if request:
            host = request.META['HTTP_HOST']
        
    if host:
        # strip the port
        host = host.split(":")[0]
        try:
            site = caching.cache_get('SITE', host=host)
        except caching.NotCachedError, nce:
            try:
                log.debug('looking up site by host: %s', host)
                site = Site.objects.get(domain=host)

            except Site.DoesNotExist:
                global _WARNED
                if not _WARNED:
                    log.warn("Site '%s' is not configured on this site - add to sites in admin", host)                
                    _WARNED = True
                
            if not site:
                site = Site.objects.get_current()
                
            caching.cache_set(nce.key, value=site)
    else:
        log.debug("No request in threadlocals, returning default site")
        site = Site.objects.get_current()
            
    #log.debug("current site: %s", site.name)
    return site
        
def site_by_request(request=None):
    """Look up a site given an explicit request rather than using the request
    currently in the `threadlocals`."""
    host = None
    if request:
        host = request.META['HTTP_HOST']
    return site_by_host(host=host)
        